import logging
from opentelemetry.exporter.otlp.proto.http._log_exporter import OTLPLogExporter
from opentelemetry.sdk.resources import Resource
from opentelemetry.instrumentation.logging import LoggingInstrumentor
from opentelemetry._logs import set_logger_provider
from opentelemetry.sdk._logs import LoggerProvider, LoggingHandler
from opentelemetry.sdk._logs.export import BatchLogRecordProcessor
from .config import get_config

# =============================================================================
# SiftDevLogger: Initializes SiftDev Logging (Singleton)
# =============================================================================
class SiftDevLogger:
    """
    This singleton initializes the SiftDev logging configuration using a SiftDevConfig.
    It sets up:
      - An OTLP log exporter (if both otlp_endpoint and sift_dev_ingest_key are provided)
      - A batching processor (with delay from the configuration)
      - A LoggingHandler attached to the root logger so that all logs that propagate are exported
      - An optional console handler for local debugging
      - Automatic context propagation via LoggingInstrumentor
    """
    _instance = None

    def __new__(cls):
        if cls._instance is None:
            cls._instance = super(SiftDevLogger, cls).__new__(cls)
            cls._instance._initialized = False
        return cls._instance

    def __init__(self):
        if self._initialized:
            return
        self._setup_logging()
        self._initialized = True

    def _setup_logging(self):
        config = get_config()
        resource = Resource.create({
            "service.name": config.service_name,
            "service.instance.id": config.service_instance_id
        })

        logger_provider = LoggerProvider(resource=resource)
        root_logger = logging.getLogger()

        if config.otlp_endpoint:
            if not config.sift_dev_ingest_key:
                warnings.warn("Sift Dev ingest key not provided. If using Sift Dev, please set the SIFT_DEV_INGEST_KEY environment variable.", stacklevel=2)
            
            otlp_headers = {
                "Content-Type": "application/json",
            }
            if config.sift_dev_ingest_key:
                otlp_headers["sift-dev-ingest-key"] = config.sift_dev_ingest_key
            
            logs_endpoint = f"{config.otlp_endpoint}/v1/logs"
            log_exporter = OTLPLogExporter(
                endpoint=logs_endpoint,
                headers=otlp_headers,
                timeout=5
            )
            
            logger_provider.add_log_record_processor(
                BatchLogRecordProcessor(log_exporter, schedule_delay_millis=config.batch_delay_millis)
            )
            set_logger_provider(logger_provider)

            otlp_handler = LoggingHandler(level=logging.NOTSET, logger_provider=logger_provider)
            root_logger.addHandler(otlp_handler)
        else:
            import warnings
            warnings.warn("OTLP endpoint not provided. OTLP logging handler will be skipped.", stacklevel=2)
            

        root_logger.setLevel(logging.INFO)

        # Instrument logging but do not add a default console handler.
        LoggingInstrumentor().instrument(set_logging_format=True)

# =============================================================================
# getLogger: Retrieve a Logger with Optional Default Attributes
# =============================================================================
def getLogger(name: str = None, extra: dict = None) -> logging.Logger:
    """
    Retrieve a logger instance configured for SiftDev logs.
    If 'extra' is provided, a LoggerAdapter is returned with those contextual attributes.
    """
    SiftDevLogger()  # Ensure logging is initialized.
    logger = logging.getLogger(name if name else __name__)
    if extra:
        logger = logging.LoggerAdapter(logger, extra)
    return logger

# =============================================================================
# flush_logs: Flush Outstanding Log Handlers
# =============================================================================
def flush_logs():
    """Flush all outstanding logs from all handlers attached to the root logger."""
    root_logger = logging.getLogger()
    for handler in root_logger.handlers:
        if hasattr(handler, "flush"):
            handler.flush()