from .common import getLogger
from .config import get_config
from fastapi import FastAPI, Request, Response
from fastapi.responses import StreamingResponse
import logging
import time
import uuid
from http import HTTPStatus
from .formatters import ColoredFormatter
import json

def instrument_logging_middleware(
    app: FastAPI,
    logger_name: str = "fastapi_access",
    max_body_size: int = 100_000,
    ignored_paths: set = {"/ping", "/health"},
    middleware_formatter: callable = None,
    console_logs_enabled: bool = True,
):
    """
    Instruments a FastAPI app with custom access logging middleware.
    This version logs the request details as soon as the request is received,
    and then logs the response details after the response body is fully processed.
    """
    access_logger = getLogger(logger_name)
    access_logger.propagate = True

    # If console logging is enabled, attach a console handler with ColoredFormatter.
    if console_logs_enabled:
        # Remove any existing console handlers
        root_logger = getLogger()
        for handler in root_logger.handlers[:]:
            if isinstance(handler, logging.StreamHandler):
                root_logger.removeHandler(handler)
        for handler in access_logger.handlers[:]:
            if isinstance(handler, logging.StreamHandler):
                access_logger.removeHandler(handler)
        console_handler = logging.StreamHandler()
        console_handler.setFormatter(ColoredFormatter())
        access_logger.addHandler(console_handler)

    def format_log_message(method: str, path: str, status_code: int, duration_ms: float) -> str:
        """Format a descriptive log message with request details"""
        try:
            status_text = HTTPStatus(status_code).phrase
        except ValueError:
            status_text = "Unknown Status"
        
        return f"{method} {path} completed with {status_code} {status_text} in {duration_ms:.2f}ms"

    def get_log_attributes(
        request_id: str,
        client_addr: str,
        request: Request,
        response: Response,
        duration_ms: float,
        request_headers: dict,
        request_body_str: str,
        response_body_str: str,
        max_body_size: int,
        error: str = None
    ) -> dict:
        """Get common log attributes, including ENV_TYPE if set"""
        attributes = {
            "request_id": request_id,
            "client_addr": client_addr,
            "route": request.url.path,
            "method": request.method,
            "status_code": response.status_code if response else 500,
            "duration_ms": duration_ms,
            "request_headers": str(request_headers),
            "response_headers": str(dict(response.headers)) if response else "",
            "request_body": request_body_str,
            "response_body": response_body_str,
            "query_params": str(dict(request.query_params)),
        }

        # Only include ENV_TYPE if it's set
        if get_config().env:
            attributes["env"] = get_config().env

        # Include error information if provided
        if error:
            attributes["error"] = error

        return attributes

    async def log_requests(request: Request, call_next):
        if request.url.path in ignored_paths:
            return await call_next(request)

        request_id = str(uuid.uuid4())
        start_time = time.time()
        client_addr = request.client.host if request.client else "unknown"
        
        # Capture request body
        try:
            body = await request.body()
            if len(body) > max_body_size:
                request_body = body[:max_body_size] + b"... (truncated)"
            else:
                request_body = body
            request_body_str = request_body.decode('utf-8', errors='replace')
        except Exception as e:
            request_body = b""
            request_body_str = ""
            access_logger.warning(f"Failed to capture request body: {str(e)}")

        try:
            response = await call_next(request)
            duration_ms = (time.time() - start_time) * 1000

            # Get response body if possible
            try:
                response_body = await response.body()
                response_body_str = response_body.decode('utf-8', errors='replace')
            except Exception:
                response_body_str = ""

            # Determine log level and capture error message if any.
            log_level = logging.ERROR if response.status_code >= 500 else logging.INFO
            error_message = ""
            if response.status_code >= 500:
                try:
                    error_obj = json.loads(response_body_str)
                    error_message = error_obj.get("error", "")
                except Exception:
                    error_message = ""

            if middleware_formatter:
                log_msg = middleware_formatter(request=request, response=response, duration_ms=duration_ms)
            else:
                log_msg = format_log_message(
                    method=request.method,
                    path=request.url.path,
                    status_code=response.status_code,
                    duration_ms=duration_ms
                )

            access_logger.log(
                log_level,
                log_msg,
                extra=get_log_attributes(
                    request_id=request_id,
                    client_addr=client_addr,
                    request=request,
                    response=response,
                    duration_ms=duration_ms,
                    request_headers=dict(request.headers),
                    request_body_str=request_body_str,
                    response_body_str=response_body_str,
                    max_body_size=max_body_size,
                    error=error_message
                )
            )
            return response
        
        except Exception as e:
            duration_ms = (time.time() - start_time) * 1000
            access_logger.error(
                format_log_message(
                    method=request.method,
                    path=request.url.path,
                    status_code=500,
                    duration_ms=duration_ms
                ),
                extra=get_log_attributes(
                    request_id=request_id,
                    client_addr=client_addr,
                    request=request,
                    response=None,
                    duration_ms=duration_ms,
                    request_headers=dict(request.headers),
                    request_body_str=request_body_str,
                    response_body_str="",
                    max_body_size=max_body_size,
                    error=str(e)
                )
            )
            raise

    app.middleware("http")(log_requests)
    return app