from flask import Flask, request, g, Response
import time
import uuid
from http import HTTPStatus
import logging
from .common import getLogger

def instrument_logging_middleware(
    app: Flask,
    logger_name: str = "flask_access",
    max_body_size: int = 100_000,
    ignored_paths: set = {"/ping", "/health"},
    middleware_formatter: callable = None,
    console_logs_enabled: bool = True,
):
    """
    Instruments a Flask app with custom access logging middleware.
    Logs request and response details using Flask standards.
    
    Args:
        app: The Flask application instance.
        logger_name: Name of the logger to use.
        max_body_size: Maximum size (in bytes) of request/response body to log.
        ignored_paths: Set of endpoint paths to ignore for logging.
        middleware_formatter: Optional callable that accepts (request, response, duration_ms)
                              and returns a formatted log message string.
        console_logs_enabled: If True, attach a console handler for local debugging.
    
    Returns:
        The modified Flask app.
    """
    access_logger = getLogger(logger_name)
    access_logger.propagate = True

    # If console logging is enabled, attach a console handler with a default formatter.
    if console_logs_enabled:
        root_logger = getLogger()
        # Remove any existing console handlers.
        for handler in root_logger.handlers[:]:
            if isinstance(handler, logging.StreamHandler):
                root_logger.removeHandler(handler)
        for handler in access_logger.handlers[:]:
            if isinstance(handler, logging.StreamHandler):
                access_logger.removeHandler(handler)
        console_handler = logging.StreamHandler()
        console_handler.setFormatter(logging.Formatter('%(asctime)s %(levelname)s %(message)s'))
        access_logger.addHandler(console_handler)

    @app.before_request
    def start_timer():
        # Skip logging for ignored paths.
        if request.path in ignored_paths:
            return
        g.start_time = time.time()
        try:
            data = request.get_data()
            if len(data) > max_body_size:
                g.request_body = data[:max_body_size] + b"... (truncated)"
            else:
                g.request_body = data
        except Exception as e:
            access_logger.warning(f"Failed to capture request body: {str(e)}")
            g.request_body = b""
        # Generate a request ID.
        g.request_id = str(uuid.uuid4())

    @app.after_request
    def log_request(response: Response):
        # Skip logging for ignored paths.
        if request.path in ignored_paths:
            return response

        duration_ms = (time.time() - g.start_time) * 1000 if hasattr(g, "start_time") else 0

        try:
            request_body = g.request_body if hasattr(g, "request_body") else b""
            request_body_str = request_body.decode('utf-8', errors="replace")
        except Exception:
            request_body_str = "<binary>"

        try:
            response_body_bytes = response.get_data()
            if len(response_body_bytes) > max_body_size:
                response_body_str = response_body_bytes[:max_body_size].decode('utf-8', errors="replace") + "... (truncated)"
            else:
                response_body_str = response_body_bytes.decode('utf-8', errors="replace")
        except Exception:
            response_body_str = "<binary>"

        extra = {
            "request_id": g.request_id if hasattr(g, "request_id") else "",
            "client_addr": request.remote_addr or "unknown",
            "route": request.path,
            "method": request.method,
            "status_code": response.status_code,
            "duration_ms": duration_ms,
            "request_headers": str(dict(request.headers)),
            "response_headers": str(dict(response.headers)),
            "request_body": request_body_str,
            "response_body": response_body_str,
            "query_params": str(request.args.to_dict())
        }

        def default_format_log_message(method, path, status_code, duration_ms):
            try:
                status_text = HTTPStatus(status_code).phrase
            except Exception:
                status_text = "Unknown Status"
            return f"{method} {path} completed with {status_code} {status_text} in {duration_ms:.2f}ms"

        if middleware_formatter:
            log_msg = middleware_formatter(request=request, response=response, duration_ms=duration_ms)
        else:
            log_msg = default_format_log_message(request.method, request.path, response.status_code, duration_ms)

        access_logger.info(log_msg, extra=extra)
        return response

    return app 