import logging
import time
from http import HTTPStatus
from typing import Dict

COLORS = {
    'RESET': '\033[0m',
    'RED': '\033[31m',
    'GREEN': '\033[32m',
    'YELLOW': '\033[33m',
    'BLUE': '\033[34m',
    'MAGENTA': '\033[35m',
    'CYAN': '\033[36m',
    'WHITE': '\033[37m',
    'GRAY': '\033[90m',
}

LEVEL_COLORS = {
    'INFO': COLORS['GREEN'],
    'WARNING': COLORS['YELLOW'],
    'ERROR': COLORS['RED'],
    'CRITICAL': COLORS['RED'],
    'DEBUG': COLORS['BLUE'],
}

def format_console_message(record: Dict) -> str:
    """Format a console message with colors and better structure."""
    status_code = record.get('status_code', 0)
    method = record.get('method', 'UNKNOWN')
    path = record.get('route', 'UNKNOWN')
    duration = record.get('duration_ms', 0)
    level = record.get('levelname', 'INFO')
    
    timestamp = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(record.get('created', time.time())))
    
    if status_code >= 500:
        status_color = COLORS['RED']
    elif status_code >= 400:
        status_color = COLORS['YELLOW']
    else:
        status_color = COLORS['GREEN']
    
    try:
        status_text = HTTPStatus(status_code).phrase
    except ValueError:
        status_text = "Unknown Status"

    method_color = {
        'GET': COLORS['BLUE'],
        'POST': COLORS['GREEN'],
        'PUT': COLORS['YELLOW'],
        'DELETE': COLORS['RED'],
        'PATCH': COLORS['MAGENTA'],
    }.get(method, COLORS['WHITE'])

    parts = []
    level_color = LEVEL_COLORS.get(level, COLORS['WHITE'])
    parts.append(f"{level_color}{level:8}{COLORS['RESET']}")
    parts.append(f"{COLORS['GRAY']}{timestamp}{COLORS['RESET']}")
    parts.append(f"{method_color}{method}{COLORS['RESET']}")
    parts.append(f"{COLORS['WHITE']}{path}{COLORS['RESET']}")    
    parts.append(f"{COLORS['MAGENTA']}{duration:.2f}ms{COLORS['RESET']}")    
    parts.append(f"{status_color}{status_code} {status_text}{COLORS['RESET']}")
    return ' '.join(parts)

class ColoredFormatter(logging.Formatter):
    def __init__(self):
        super().__init__()

    def format(self, record):
        if hasattr(record, 'status_code'):
            if hasattr(record, 'extra'):
                for key, value in record.extra.items():
                    setattr(record, key, value)
            return format_console_message(record.__dict__)
        return super().format(record) 